import { dv as Json, dw as ObjectStorage } from './uuid-d1a61975.js';
import { g as LogHandlerOptions, c as LogMessage, d as LogHandler } from './log-b80164f3.js';

/** Copy string to clipboard */
declare function pbcopy(data: string): Promise<unknown>;

declare type csvOptions = {
    /** @deprecated will probably be replaced by logLevel */
    debug?: boolean;
    path?: string;
    filename?: string;
    encoding?: BufferEncoding;
    prefix?: string;
    env?: Record<string, string>;
};
/**
 * Return a path relative to the current working directory
 */
declare function stringToPath(value?: string, defaultValue?: string): string;
declare function valueToPath(value?: any, defaultValue?: string): string;
declare const toPath: typeof valueToPath;
declare function setupEnv(options?: csvOptions): {
    parsed: Record<string, string>;
    error?: undefined;
} | {
    error: unknown;
    parsed?: undefined;
};

interface FileStorageOptions {
    pretty?: boolean;
    path?: string;
    extension?: string;
    objectFromString?: (data: string) => any;
    objectToString?: (data: any) => string;
    keyToFilename?: (key: string) => string;
}
declare class FileStorage<T = Json> implements ObjectStorage<T> {
    private store;
    private dirname;
    private fileKeys?;
    private pretty;
    private extension;
    private extensionLength;
    private objectFromString;
    private objectToString;
    private keyToFilename;
    constructor(opt?: FileStorageOptions);
    setItem(key: string, value: T): void;
    getPath(key: string): string;
    getBuffer(key: string): Buffer;
    getItem(key: string): T | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}

declare function exists(path: string): Promise<boolean>;
declare function ensureFolder(...parts: string[]): Promise<string>;
declare function removeFolder(...parts: string[]): Promise<string>;
declare function readText(...parts: string[]): Promise<string | undefined>;
declare function writeText(path: string, content: string): Promise<void>;

declare function LoggerFileHandler(path: string, opt?: LogHandlerOptions): (msg: LogMessage) => void;

declare function colorString(value: string, colorCode: number): string;
declare function colorStringList(list: Array<any>, style: string, bold?: boolean): any[];
declare const loggerStackTraceDebug = "loggerStackTraceDebug-7d38e5a9214b58d29734374cdb9521fd964d7485";
declare function LoggerNodeHandler(opt?: LogHandlerOptions): LogHandler;

declare function getStackLlocationList(stack: string): any[];
/**
 * Get the source code location of the caller
 * https://stackoverflow.com/a/47296370/140927
 *
 * @param level Number of levels to go down the stack trace
 * @param stripCwd Strip the current working directory, only reasonable for Node.js environment
 * @returns
 */
declare function getSourceLocation(level?: number, stripCwd?: boolean): string;
declare function getStack(): string;
declare function getSourceLocationByPrecedingPattern(patterns: string[], stripCwd?: boolean): string;

export { FileStorageOptions as F, LoggerFileHandler as L, setupEnv as a, FileStorage as b, ensureFolder as c, readText as d, exists as e, colorString as f, colorStringList as g, LoggerNodeHandler as h, getStackLlocationList as i, getSourceLocation as j, getStack as k, loggerStackTraceDebug as l, getSourceLocationByPrecedingPattern as m, pbcopy as p, removeFolder as r, stringToPath as s, toPath as t, valueToPath as v, writeText as w };
